/*
 * LCD_I2C.c
 *
 * Created: 2011/12/11 10:39:30
 *  Author: Satoshi
 */ 

#include "cpu_def.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/delay.h>


#include "Lcd.h"
#include "TWI_Slave.h"
#include "TWI_DEF.h"

#define LCD_ADDRESS 0x20								// TWI(I2C) Address

unsigned char TwiGetReciveData( void )
{
	unsigned char data = (unsigned char)0x00;
	unsigned char messageBuf[TWI_BUFFER_SIZE];

	if ( ! TWI_Transceiver_Busy() ) {					// Check if the last operation was successful
		if ( TWI_statusReg.lastTransOK ) {				// Check if the last operation was a reception
			if ( TWI_statusReg.RxDataInBuf ) {
				TWI_Get_Data_From_Transceiver(messageBuf, 2);         
				if ( TWI_statusReg.genAddressCall ) {	// Check if the last operation was a reception as General Call
					data = (unsigned char)0x00;
				}               
				else {									// Ends up here if the last operation was a reception as Slave Address Match   
           			if (messageBuf[0] == TWI_CMD_MASTER_WRITE) {	// TWI_CMD_MASTER_WRITE
						data = messageBuf[1];                            
					}
					if (messageBuf[0] == TWI_CMD_MASTER_READ) {		// TWI_CMD_MASTER_READ
						messageBuf[0] = (unsigned char)0x00;
						TWI_Start_Transceiver_With_Data( messageBuf, 1 );
					}
				}
			}                
			else { 										// Ends up here if the last operation was a transmission  
 //     	       __no_operation(); 					// Put own code here.
 			}
			if ( ! TWI_Transceiver_Busy() ) {			// Check if the TWI Transceiver has already been started.
				TWI_Start_Transceiver();				// If not then restart it to prepare it for new receptions.
			}
		}
	}
	return data;
}


int main(void)
{
	unsigned char TWI_slaveAddress;

	int8_t  x, y, n1, n2;
	uint8_t esc;
	char    d;

	DDRB  = 0b11111111;		// 7 - 0:data 
	DDRD  = 0b11111100;		// 7:LED 6:RS 5:E   4:   3:   2:   1:    0: 
	PORTB = 0b11111111;
	PORTD = 0b11111100;	

	_delay_ms( 10 );		// Wait

	TWI_slaveAddress = LCD_ADDRESS;		// own TWI slave address
										// Initialise TWI module for slave operation. Include address and/or enable General Call.
	TWI_Slave_Initialise( (unsigned char)((TWI_slaveAddress<<TWI_ADR_BITS) | (TRUE<<TWI_GEN_BIT) ));

    sei();								// S荞݋

	wdt_enable(WDTO_500MS);				// Enable of Watch dog timer 500mSec
	LcdTextInitialize();				// Initialise LCD module
	TWI_Start_Transceiver();

	x = y = 0;
	LcdTextClear( );
	LcdTextLED ( LCD_DISPLAY_ON );

	LcdTextPutstr( "Ready." );

	esc = 0;
	n1 = n2 = 0;
    while (1) {
        d = TwiGetReciveData();
		if (d != 0) {
			if (esc == 0) {
				switch( d ) {
				case '\r':					// CR
					x = 0;
					LcdTextLocate(x, y);
					break;
				case '\n':					// LF
					y+=1;
					if (y > LCD_MAX_Y) y = LCD_MAX_Y;
					LcdTextLocate(x, y);
					break;
				case '\b':					// BS
					x-= 1;
					if (x < 0) x = 0;
					LcdTextLocate( x, y );
					LcdTextPutch( ' ' );
					LcdTextLocate( x, y );
					break;
				case '\f':					// FF (Clear)
					x = y = 0;
					LcdTextClear( );
					break;
				case (char)0x11:			// LED On
					LcdTextLED( LCD_LED_ON );
					break;
				case (char)0x12:			// LED Off
					LcdTextLED( LCD_LED_OFF );
					break;
				case (char)0x1b:			// ESC
					esc = 1;
					break;
				default:
					if (d >= 0x20) {
						LcdTextPutch( d );
						x += 1;
						if (x > LCD_MAX_X) {
							x  = 0;
							y += 1;
							if (y > LCD_MAX_Y) y = LCD_MAX_Y;
						}
					}
					break;
				}
			}
			else if (esc == 1) {
				if (d == '[') esc = 2;
				else          esc = 0;
			}
			else if (esc == 2) {
				if (d == '>') esc = 10;
				else {
					n1 = (uint8_t)d - (uint8_t)'0';
					esc = 3;
				}
			}
			else if (esc == 3) {
				if (d >= '0' && d <= '9') {
					n1 = n1 * 10 + (uint8_t)d - (uint8_t)'0';
				}
				else if (d == ';') {					// Locate
					esc = 4;
				}
				else {
					if (d == 'A') {					// Cursor n Up
						y = y - n1;
						if (y < 0) y = 0;
					}
					if (d == 'B' ) {				// Cursor n Down
						y = y + n1;
						if (y > LCD_MAX_Y) y = LCD_MAX_Y;
					}
					if (d == 'C' ) {				// Cursor n Left
						x = x - n1;
						if (x < 0) x = 0;
					}
					if (d == 'D' ) {				// Cursor n Right
						x = x + n1;
						if (x > LCD_MAX_X) x = LCD_MAX_X;
					}
					LcdTextLocate(x, y);
					esc = 0;
				}
			}
			else if (esc == 4) {				// Locate
				n2 = (uint8_t)d - (uint8_t)'0';
				esc = 5;
			}
			else if (esc == 5) {				// Locate
				if (d >= '0' && d <= '9') {
					n2 = n2 * 10 + (uint8_t)d - (uint8_t)'0';
				}
				else if (d == 'H') {
					n1 -= 1;
					n2 -= 1;
					if (n1 < 0 ) n1 = 0;
					if (n2 < 0 ) n2 = 0;
					if (n1 > LCD_MAX_Y) n1 = LCD_MAX_Y;
					if (n2 > LCD_MAX_X) n2 = LCD_MAX_X;
					x = n2;
					y = n1;
					LcdTextLocate(x, y);
					esc = 0;
				}
				else esc = 0;
			}
			else if (esc == 10) {
				switch( d ) {
				case '1':
					esc = 11;
					break;
				case '3':
					esc = 13;
					break;
				case '5':
					esc = 15;
					break;
				case '6':
					esc = 16;
					break;
				default:
					esc = 0;
				
				}
			}
			else if (esc == 11) {		// Cursor Brink
				if (d == 'l')       LcdTextMode( LCD_BRINK_ON );
				else if (d == 'h' ) LcdTextMode( LCD_BRINK_OFF );
				esc = 0;
			}
			else if (esc == 13) {		// Shift Left/Right
				if (d == 'l')       LcdTextCursor( LCD_CURSOR_RIGHT_D );
				else if (d == 'h' ) LcdTextCursor( LCD_CURSOR_LEFT_D );
				esc = 0;
			}
			else if (esc == 15) {		// Cursor shift
				if (d == 'l')       LcdTextMode( LCD_CURSOR_ON );
				else if (d == 'h' ) LcdTextMode( LCD_CURSOR_OFF );
				esc = 0;
			}
			else if (esc == 16) {		// Cursor shift Left/Right
				if (d == 'l')       LcdTextCursor( LCD_CURSOR_RIGHT );
				else if (d == 'h' ) LcdTextCursor( LCD_CURSOR_LEFT );
				esc = 0;
			}
			else esc = 0;
		}

		wdt_reset();									// Reset of Watch dog timer
    }
}